# Example 7.12 of 
# Mehttalo, L. and Lappi, J. 2020. Biometry for forestry and environmental data: With examples in R. CRC Press. 
# 
# Author: Lauri Mehttalo
###############################################################################

# Compute the value of the Richards curve
# and the gradients wrt all three parameters to attribute gradient.
# This function could be used with nlme to estimate model 
# with user-supplied gradients.
f<-function(age,phi1,phi2,phi3) {
   value<-phi1*(1-exp(-phi3*age))^phi2
   grad<-cbind((1-exp(-phi3*age))^phi2,
                phi1*(1-exp(-phi3*age))^phi2*log(1-exp(-phi3*age)),
                phi1*phi2*(1-exp(-phi3*age))^(phi2-1)*exp(-phi3*age)*age)
   attr(value,"gradient") <- grad
   value
   }
   
library(magic)
   
D1<-diag(c(9.1^2,0.289^2))
D1[1,2]<-D1[2,1]<-9.1*0.289*0.187

D2<-diag(c(4.4^2,0.084^2))
D2[1,2]<-D2[2,1]<-4.4*0.084*0.482

T1<-c(10,15); H1<-c(43,58)
T2<-15; H2<-c(53)
T<-c(T1,T2); H<-c(H1,H2)

D<-adiag(D1,D2,D2)

sigma2<-2.16^2
R<-diag(rep(sigma2,3))

Aijk<-diag(rep(1,3))
A<-rbind(Aijk,Aijk,Aijk)
Bijk<-cbind(c(1,0,0),c(0,1,0))
zeroblock<-matrix(0,ncol=2,nrow=3)
B1<-rbind(Bijk,Bijk,Bijk)
B2<-cbind(rbind(Bijk,Bijk,zeroblock),rbind(zeroblock,zeroblock,Bijk))
B<-cbind(B1,B2)
beta<-c(86.2,1.385,0.0702)

# Initial guesses for random effects
b0<-rep(0,6)   # pool the random effects to b0
bOld<-b0-1     # initialize for the loop
history<-matrix(ncol=6,nrow=0)

while (sum((b0-bOld))^2>1e-10) {
#for (i in 1:5) {
       phi<-A%*%beta+B%*%b0
       phi1<-phi[seq(1,9,by=3)]
       phi2<-phi[seq(2,9,by=3)]
       phi3<-phi[seq(3,9,by=3)]
       pred<-f(T,phi1,phi2,phi3)
       X<-attributes(pred)$gradient
       Za<-X[,c(1,2)]
       Zb<-adiag(Za[1:2,],matrix(Za[3,],ncol=2))
       Z<-cbind(Za,Zb)
       bOld<-b0
       b0<-D%*%t(Z)%*%solve(Z%*%D%*%t(Z)+R)%*%(H-pred+Z%*%b0)
       history<-rbind(history,t(b0))
       cat(b0,"\n")
       }

b<-b0
cat(b,"\n")
		
age<-seq(0,25,0.1)
predFixed<-f(age,beta[1],beta[2],beta[3])
predPlot<-f(age,beta[1]+b[1],beta[2]+b[2],beta[3])
predTree1<-f(age,beta[1]+b[1]+b[3],beta[2]+b[2]+b[4],beta[3])
predTree2<-f(age,beta[1]+b[1]+b[5],beta[2]+b[2]+b[6],beta[3])

sePlot<-(D-D%*%t(Z)%*%solve(Z%*%D%*%t(Z)+R)%*%Z%*%D)[1:2,1:2]
Z0<-attributes(predPlot)$gradient[,1:2]
sePred<-sqrt(diag(Z0%*%sePlot%*%t(Z0)))
ubPlot<-predPlot+1.96*sePred
lbPlot<-predPlot-1.96*sePred

# Site index and confidence bounds
print(SI<-predPlot[age==25])
SI+c(-1.96,1.96)*sePred[age==25]

# SI after the first iteration
# Plug the predicted RE after first iteration 
# to the linearized model, see Meng and Huang (2009) for justification.
b0<-history[1,] # predicted RE after first iteration
predf<-f(25,beta[1],beta[2],beta[3])
predf+attributes(predf)$gradient[,c(1,2)]%*%b0[c(1,2)]

windows(width=5,height=5)
par(mai=c(0.6,0.6,0.1,0.1),mgp=c(2,0.7,0))
plot(age,predFixed,lwd=3,type="l",xlab="Age (years)",ylab="Height (feet)",ylim=c(0,80),lty="dashed")
points(T,H,pch=c(15,15,16))
lines(age,predPlot,lwd=3)
lines(age,lbPlot,lty="dashed")
lines(age,ubPlot,lty="dashed")
lines(age,predTree1)
lines(age,predTree2)
legend("topleft",lty=c("dashed","solid","dashed","solid"),lwd=c(3,3,1,1),
		         legend=c("Fixed part only","Plot-level mean curve",
				 "95% CI for plot-level mean","Tree-level curves"))



 