# Examples 7.4-7.6 of
# Mehttalo, L. and Lappi, J. 2020. Biometry for forestry and environmental data: With examples in R. CRC Press. 
# 
# Author: Lauri Mehttalo and Juha Lappi
###############################################################################

library(lmfor)
data(foto)

LightResp<-function(PPFD,Pmax=10,alpha=0.1,A0=0) {
	       A0+Pmax*PPFD/(alpha+PPFD)
           }

model5<-nlme(A~LightResp(PARtop,Pmax,alpha,A0),
		fixed=list(Pmax~Site+Treatment,alpha~Site+Treatment+moisture,A0~Site),
		random=list(sample=Pmax+alpha+A0~1),
		data=foto,
		start=c(c(100,0,0,0,0,0,0),c(80,0,0,0,0,0,0,0),c(-20,0,0,0,0,0)),
		verbose=TRUE)
D<-model5$sigma^2*pdMatrix(model5$modelStruct$reStruct)$sample
R<-cov2cor(D)

E<-eigen(R)
lambda<-E$values
Q<-E$vectors

phi0<-fixef(model5)[c(1,8,16)]
lty<-c("dashed","dotted","dotdash")
PPFD<-seq(0,800)

windows(width=5,height=5)
par(mai=c(0.6,0.6,0.1,0.1),mgp=c(2,0.7,0))
plot(PPFD,LightResp(PPFD,phi0[1],phi0[2],phi0[3]),type="l",ylim=c(-20,45),ylab="photosynthesis",xlab="x")
for (i in 1:3) {
    phi1<-phi0+sqrt(lambda[i]*diag(D))*Q[,i]
	phi2<-phi0-sqrt(lambda[i]*diag(D))*Q[,i]
	lines(PPFD,LightResp(PPFD,phi1[1],phi1[2],phi1[3]),lty=lty[i])
	lines(PPFD,LightResp(PPFD,phi2[1],phi2[2],phi2[3]),lty=lty[i])
}
legend("bottomright",legend=c("PC1","PC2","PC3"),lty=lty)

D
R
E
