# Examples 7.7-7.9 of
# Mehttalo, L. and Lappi, J. 2020. Biometry for forestry and environmental data: With examples in R. CRC Press. 
# 
# Author: Lauri Mehttalo
###############################################################################

### Example 7.7
set.seed(12345)
beta<-0.5
b<-rnorm(5,0,0.15)
phi<-beta+rep(b,each=5)
x<-rep(seq(0,8,2),5)
group<-rep(1:5,each=5)
y<-phi^x+rnorm(25,0,0.05)
nlmeData<-data.frame(x,y,group)

windows(width=5,height=5)
par(mai=c(0.6,0.6,0.1,0.1),mgp=c(2,0.7,0))
library(lmfor)
linesplot(x,y,group,xlab=expression(x[ij]),ylab=expression(y[ij]))
###

### Example 7.8

fun<-function(x,phi) phi^x
groupmeans<-with(nlmeData[nlmeData$x<=2,],tapply(y,group,mean))
mod<-nlme(y~fun(x,phi),
		fixed=phi~1,
		random=phi~1|group,
		start=mean(groupmeans))
summary(mod)
###

### Example 7.9
# Starting values
b0<-rep(0,5)
groupmeans<-with(nlmeData[nlmeData$x<=2,],tapply(y,group,mean))
beta0<-mean(groupmeans)
D<-var(groupmeans)
s2<-rep(NA,5)
for (i in 1:5) {
     s2[i]<-summary(nls(y~phi^x,start=list(phi=beta0),data=nlmeData[nlmeData$group==i,]))$sigma^2
}
sigma2<-mean(s2)
L<-chol(D/sigma2)
Delta0<-diag(rep(solve(L),5))

# Matrices A and B and the pseudo-data for the PNLS-step.
A<-matrix(1,ncol=1,nrow=25)
Bi<-matrix(1,ncol=1,nrow=5)
library(magic)
B<-adiag(Bi,Bi,Bi,Bi,Bi)
y<-nlmeData$y
x<-nlmeData$x
group<-nlmeData$group
ypseudo<-c(y,rep(0,5))

betaold<-0 # initialize to control the stopping criteria
i<-0

while ((beta0-betaold)^2>1e-16) {
       i<-i+1
       fn<-function(x,beta,b1,b2,b3,b4,b5) {
           b<-c(b1,b2,b3,b4,b5)
           phi<-A%*%beta+B%*%b
           c(fun(x,phi),Delta0%*%b)
           }

       pnlsmod<-nls(ypseudo~fn(x,beta,b1,b2,b3,b4,b5),
                    start=list(beta=beta0,b1=b0[1],b2=b0[2],b3=b0[3],b4=b0[4],b5=b0[5]))
   
       betaold<-beta0
       beta0<-coef(pnlsmod)[1]
       b0<-coef(pnlsmod)[-1]
       cat("PNLS-step",i,": beta:",beta0,"\n")
       cat("PNLS-step",i,": b:",b0,"\n")
       phi0<-A%*%beta0+B%*%b0
       X1<-matrix(x*phi0^(x-1),ncol=1)
       z<-x*phi0^(x-1)
       Z1<-matrix(0,ncol=5,nrow=25)
       for (j in 1:5) Z1[group==j,j]<-z[group==j]

       w1<-y-fun(x,phi0)+X1%*%beta0+Z1%*%b0
       lmemod<-lme(w1~X1-1,random=~z-1|group,method="ML")
       L<-chol(getVarCov(lmemod)/lmemod$sigma^2)
       Delta0<-diag(rep(solve(L),5))
       beta0<-fixef(lmemod)
       cat("LME-step",i,": sigmab^2 and sigma^2",sqrt(getVarCov(lmemod)), lmemod$sigma,"\n")
}

round(summary(pnlsmod)$parameters[1,],5)
VarCorr(lmemod)
coef(pnlsmod)[-1]

### prediction of random effects (not in the textbook)
# A corresponding example with a two-level model is in Example 7.12
y
x
group

summary(mod)

beta<-0.558
D<-0.0671^2
sigma2<-0.0506^2

dat<-data.frame(x,y,group)

caldat<-dat[dat$group==2,]
caldat
#[c(2,4),]

# Predict b using Taylor approximation around b*=0

y<-caldat$y
mu<-beta^caldat$x
Z<-matrix(caldat$x*beta^(caldat$x-1),ncol=1)
R<-diag(rep(sigma2,dim(caldat)[1]))

b<-D%*%t(Z)%*%solve(Z%*%D%*%t(Z)+R)%*%(y-mu)
b
mu+Z%*%b

# Iterate: make the aproximation around current b
b<-bvec<-0
yhatmat<-c()
for (i in 1:10) {
y<-caldat$y
mu<-(beta+b)^caldat$x-caldat$x*(beta+b)^(caldat$x-1)*b
Z<-matrix(caldat$x*(beta+b)^(caldat$x-1),ncol=1)
R<-diag(rep(sigma2,dim(caldat)[1]))
b<-D%*%t(Z)%*%solve(Z%*%D%*%t(Z)+R)%*%(y-mu)
bvec<-c(bvec,b)
yhatmat<-cbind(yhatmat,mu+Z%*%b)
}

windows()
plot(caldat$x,caldat$y)
lines(caldat$x,yhatmat[,1],lty="dashed")
lines(caldat$x,yhatmat[,2],col="red",lwd=2)
lines(caldat$x,yhatmat[,10],col="blue")