# Examples 7.10 and 7.11 of
# Mehttalo, L. and Lappi, J. 2020. Biometry for forestry and environmental data: With examples in R. CRC Press. 
# 
# Author: Lauri Mehttalo
###############################################################################
# Note: the published book includes slightly different results than these scripts. 
# Most likely the  data in those examples has been slightly different

### Example 7.10

# Reparameterized logistic function
mylogis<-function(x,phi1,phi2) {
         ephi1<-exp(phi1)
         ephi2<-exp(phi2)
         ephi1/(1+exp(4-8*x/ephi2))
}

library(lmfor)
data(thefdata)
thefdata$ythin<-thefdata$Year-1986
thefdata$diamS<-(thefdata$Diam1986-mean(thefdata$Diam1986))/sd(thefdata$Diam1986)
ThinNlme0<-nlme(ThEf~mylogis(ythin,phi1,phi2),
                fixed=list(phi1~1,phi2~1),
                random=phi1+phi2~1|Tree,
                data=thefdata,
                start=c(log(200),log(5)),
                verbose=TRUE,
                control=list(pnlsMaxIter=20))

re<-ranef(ThinNlme0,aug=TRUE)[,c(1,2,3,5,7,10)]
head(re)

re$SDClass<-as.factor(as.character(re$SDClass))

windows(width=7,height=7)
par(mfcol=c(2,2),mai=c(0.7, 0.6, 0.1, 0.1),mgp=c(2, 0.7, 0))
plot(re$SDClass,re$phi1,xlab="Thinning treatment",ylab=expression(tilde(b)[i]^(1)))
plot(re$SDClass,re$phi2,xlab="Thinning treatment",ylab=expression(tilde(b)[i]^(2)))
plot(re$diamS,re$phi1,xlab="Standardized initial diameter",ylab=expression(tilde(b)[i]^(1)))
plot(re$diamS,re$phi2,xlab="Standardized initial diameter",ylab=expression(tilde(b)[i]^(2)))

beta0<-fixef(ThinNlme0)
ThinNlme1<-nlme(ThEf~mylogis(ythin,phi1,phi2),
                fixed=list(phi1~SDClass,phi2~diamS),
                random=phi1+phi2~1|Tree,
                data=thefdata,
                start=c(beta0[1],0,0,beta0[2],0),
                control=list(pnlsMaxIter=20))
summary(ThinNlme1)

windows()
plot(fitted(ThinNlme1),resid(ThinNlme1))
mywhiskers(fitted(ThinNlme1),resid(ThinNlme1),add=TRUE,se=FALSE)
abline(0,0)

windows()
plot(ACF(ThinNlme1),alpha=0.05)


ThinNlme2<-update(ThinNlme1,
           weights=varPower(),
           cor=corAR1(),
           verbose=TRUE,
           control=nlmeControl(msVerbose=TRUE))

windows()
plot(fitted(ThinNlme2),resid(ThinNlme2,type="p"))
mywhiskers(fitted(ThinNlme2),resid(ThinNlme2,type="p"),add=TRUE,se=FALSE)
abline(0,0)

windows()
plot(ACF(ThinNlme2,resType="n"),alpha=0.05)

anova(ThinNlme2,ThinNlme1)

anova(ThinNlme2,type="m")

ThinNlme3<-update(ThinNlme2,
                  fixed=list(phi1~SDClass+diamS,phi2~SDClass+diamS),
                  start=c(beta0[1],0,0,0,beta0[2],0,0,0))
anova(ThinNlme3,type="m")


summary(ThinNlme2)
exp(fixef(ThinNlme2))
###


### Example 7.11
ThinNlme4<-update(ThinNlme2,
                  random=phi1+phi2~1|Plot/Tree,
                  start=fixef(ThinNlme2))
summary(ThinNlme4)
anova(ThinNlme2,ThinNlme4)
###